@@ -3,15 +3,26 @@
 """This module defines the `Style` class.
 """
 
+from __future__ import annotations
+
 # These are for typing our methods:
-from typing import List, Tuple, Union
+from typing import List, Tuple, Union, TYPE_CHECKING
+
 
 # These ones are internal to `violetear`:
 from .selector import Selector
-from .units import GridSize, GridTemplate, Unit, fr, ms, pc, minmax, rem, repeat, sec
+from .units import Unit, fr, ms, pc, minmax, rem, repeat, sec
+from .types import GridSize, GridTemplate, FontWeight
 from .color import Color, gray
 from .helpers import style_method
 
+# This trick is necessary to annotate the `Style.animation` method
+# without incurring in cyclic import errors,
+# because the `Animation` class does need to import `Style` in runtime.
+
+if TYPE_CHECKING:
+    from .animation import Animation
+
 # And this one is for generating CSS:
 import textwrap
 
@@ -30,7 +41,7 @@ import textwrap
 
 class Style:
     def __init__(
-        self, selector: Union[str, Selector] = None, *, parent: "Style" = None
+        self, selector: Union[str, Selector] = None, *, parent: Style = None
     ) -> None:
         """Create a new instance of `Style`.
 
@@ -50,6 +61,8 @@ class Style:
         self._children = {}
         self._transitions = []
         self._transforms = {}
+        self._animations = set()
+        self._animation_configs = []
 
     # ### Basic rule manipulation
     # These methods allows manipulating rules manually.
@@ -60,7 +73,7 @@ class Style:
     # but complex values like `Unit` and `Color` will be converted to their string representation
     # using their corresponding `__str__` methods.
 
-    def rule(self, attr: str, value) -> "Style":
+    def rule(self, attr: str, value) -> Style:
         """Define a new CSS rule.
 
         **Parameters**:
@@ -73,8 +86,11 @@ class Style:
 
     # #### `Style.rules`
 
-    def rules(self, **rules) -> "Style":
-        """Define a bunch of CSS rules at once with kwargs."""
+    def rules(self, **rules) -> Style:
+        """Define a bunch of CSS rules at once with kwargs.
+
+        Attribute names are automatically converted from `snake_case` to `kebab-case`.
+        """
 
         for rule, value in rules.items():
             self.rule(rule.replace("_", "-"), value)
@@ -94,7 +110,7 @@ class Style:
     # However, in favour of YAGNI, I will refrain from implementing that functionality until
     # proven necessary.
 
-    def apply(self, *others: "Style") -> "Style":
+    def apply(self, *others: Style) -> Style:
         """Copy rules from other styles.
 
         Rules are defined in order, so later styles will override former ones.
@@ -113,12 +129,16 @@ class Style:
     # These methods allow manipulating font and text properties.
 
     # #### `Style.font`
-    # The `font` method
+    # Configure font attributes.
 
     @style_method
     def font(
-        self, size: Unit = None, *, weight: str = None, family: str = None
-    ) -> "Style":
+        self,
+        size: Unit = None,
+        *,
+        weight: FontWeight = None,
+        family: str = None,
+    ) -> Style:
         if size:
             self.rule("font-size", Unit.infer(size))
 
@@ -129,9 +149,10 @@ class Style:
             self.rule("font-family", family)
 
     # #### `Style.text`
+    # Configure text styling attributes.
 
     @style_method
-    def text(self, *, align: str = None, decoration: str = None) -> "Style":
+    def text(self, *, align: str = None, decoration: str = None) -> Style:
         if align is not None:
             self.rule("text-align", align)
         if decoration is not None:
@@ -140,27 +161,31 @@ class Style:
             self.rule("text-decoration", decoration)
 
     # #### `Style.center`
+    # Shorthand method for center align.
 
     @style_method
-    def center(self) -> "Style":
+    def center(self) -> Style:
         self.text(align="center")
 
     # #### `Style.left`
+    # Shorthand method for left align.
 
     @style_method
-    def left(self) -> "Style":
+    def left(self) -> Style:
         self.text(align="left")
 
     # #### `Style.right`
+    # Shorthand method for right align.
 
     @style_method
-    def right(self) -> "Style":
+    def right(self) -> Style:
         self.text(align="right")
 
     # #### `Style.justify`
+    # Shorthand method for justified align.
 
     @style_method
-    def justify(self) -> "Style":
+    def justify(self) -> Style:
         self.text(align="justify")
 
     # ### Color styles
@@ -168,39 +193,13 @@ class Style:
     # #### `Style.color`
 
     @style_method
-    def color(
-        self, color: Color = None, *, rgb=None, hsv=None, hls=None, alpha: float = None
-    ) -> "Style":
-        if color is None:
-            if rgb is not None:
-                r, g, b = rgb
-                color = Color(r, g, b, alpha)
-            elif hsv is not None:
-                h, s, v = hsv
-                color = Color.from_hsv(h, s, v, alpha)
-            elif hls is not None:
-                h, l, s = hls
-                color = Color.from_hls(h, l, s, alpha)
-
+    def color(self, color: Color) -> Style:
         self.rule("color", color)
 
     # #### `Style.background`
 
     @style_method
-    def background(
-        self, color: Color = None, *, rgb=None, hsv=None, hls=None, alpha: float = None
-    ) -> "Style":
-        if color is None:
-            if rgb is not None:
-                r, g, b = rgb
-                color = Color(r, g, b, alpha)
-            elif hsv is not None:
-                h, s, v = hsv
-                color = Color.from_hsv(h, s, v, alpha)
-            elif hls is not None:
-                h, l, s = hls
-                color = Color.from_hls(h, l, s, alpha)
-
+    def background(self, color: Color) -> Style:
         self.rule("background-color", color)
 
     # #### `Style.border`
@@ -208,7 +207,7 @@ class Style:
     @style_method
     def border(
         self, width: Unit = None, color: Color = None, *, radius: Unit = None
-    ) -> "Style":
+    ) -> Style:
         if width is not None:
             self.rule("border-width", Unit.infer(width))
 
@@ -216,26 +215,26 @@ class Style:
             self.rule("border-color", color)
 
         if radius is not None:
-            self.rule("border-color", Unit.infer(radius))
+            self.rule("border-radius", Unit.infer(radius))
 
     # ### Visibility styles
 
     # #### `Style.visibility`
 
     @style_method
-    def visibility(self, visibility: str) -> "Style":
+    def visibility(self, visibility: str) -> Style:
         self.rule("visibility", visibility)
 
     # #### `Style.visible`
 
     @style_method
-    def visible(self) -> "Style":
+    def visible(self) -> Style:
         self.visibility("visible")
 
     # #### `Style.hidden`
 
     @style_method
-    def hidden(self) -> "Style":
+    def hidden(self) -> Style:
         self.visibility("hidden")
 
     # ### Geometry styles
@@ -243,7 +242,7 @@ class Style:
     # #### `Style.width`
 
     @style_method
-    def width(self, value=None, *, min=None, max=None) -> "Style":
+    def width(self, value=None, *, min=None, max=None) -> Style:
         if value is not None:
             self.rule("width", Unit.infer(value, on_float=pc))
 
@@ -256,7 +255,7 @@ class Style:
     # #### `Style.height`
 
     @style_method
-    def height(self, value=None, *, min=None, max=None) -> "Style":
+    def height(self, value=None, *, min=None, max=None) -> Style:
         if value is not None:
             self.rule("height", Unit.infer(value, on_float=pc))
 
@@ -266,12 +265,25 @@ class Style:
         if max is not None:
             self.rule("max-height", Unit.infer(max, on_float=pc))
 
+    # #### `Style.size`
+
+    @style_method
+    def size(self, width:Unit, height:Unit) -> Style:
+        self.width(width)
+        self.height(height)
+
     # #### `Style.margin`
 
     @style_method
     def margin(
-        self, all=None, *, left=None, right=None, top=None, bottom=None
-    ) -> "Style":
+        self,
+        all=None,
+        *,
+        left=None,
+        right=None,
+        top=None,
+        bottom=None,
+    ) -> Style:
         if all is not None:
             self.rule("margin", Unit.infer(all))
         if left is not None:
@@ -287,8 +299,14 @@ class Style:
 
     @style_method
     def padding(
-        self, all=None, *, left=None, right=None, top=None, bottom=None
-    ) -> "Style":
+        self,
+        all=None,
+        *,
+        left=None,
+        right=None,
+        top=None,
+        bottom=None,
+    ) -> Style:
         if all is not None:
             self.rule("padding", Unit.infer(all))
         if left is not None:
@@ -303,7 +321,7 @@ class Style:
     # #### `Style.rounded`
 
     @style_method
-    def rounded(self, radius: Unit = None) -> "Style":
+    def rounded(self, radius: Unit = None) -> Style:
         if radius is None:
             radius = 0.25
 
@@ -314,7 +332,7 @@ class Style:
     # #### `Style.display`
 
     @style_method
-    def display(self, display: str) -> "Style":
+    def display(self, display: str) -> Style:
         self.rule("display", display)
 
     # #### `Style.flexbox`
@@ -324,12 +342,12 @@ class Style:
         self,
         direction: str = "row",
         *,
-        gap: Unit = None,
+        gap: Unit = 0,
         wrap: bool = False,
         reverse: bool = False,
         align: str = None,
         justify: str = None,
-    ) -> "Style":
+    ) -> Style:
         self.display("flex")
 
         if reverse:
@@ -346,8 +364,7 @@ class Style:
         if justify is not None:
             self.rule("justify-content", justify)
 
-        if gap is not None:
-            self.rule("gap", Unit.infer(gap))
+        self.rule("gap", Unit.infer(gap))
 
     # #### `Style.flex`
 
@@ -357,7 +374,7 @@ class Style:
         grow: float = None,
         shrink: float = None,
         basis: int = None,
-    ) -> "Style":
+    ) -> Style:
         if grow is not None:
             self.rule("flex-grow", float(grow))
 
@@ -378,7 +395,7 @@ class Style:
         auto_columns: GridSize = None,
         auto_rows: GridSize = None,
         gap: Unit = 0,
-    ) -> "Style":
+    ) -> Style:
         self.display("grid")
 
         if columns is None and rows is None:
@@ -406,8 +423,13 @@ class Style:
 
     @style_method
     def columns(
-        self, count: int, min: GridSize = None, max: GridSize = None, *, gap: Unit = 0
-    ) -> "Style":
+        self,
+        count: int,
+        min: GridSize = None,
+        max: GridSize = None,
+        *,
+        gap: Unit = 0,
+    ) -> Style:
         if min is None:
             min = fr(1)
 
@@ -420,8 +442,13 @@ class Style:
 
     @style_method
     def rows(
-        self, count: int, min: GridSize = None, max: GridSize = None, *, gap: Unit = 0
-    ) -> "Style":
+        self,
+        count: int,
+        min: GridSize = None,
+        max: GridSize = None,
+        *,
+        gap: Unit = 0,
+    ) -> Style:
         if min is None:
             min = fr(1)
 
@@ -437,7 +464,7 @@ class Style:
         self,
         columns: Union[int, Tuple[int, int]] = None,
         rows: Union[int, Tuple[int, int]] = None,
-    ) -> "Style":
+    ) -> Style:
         if columns is not None:
             if isinstance(columns, tuple):
                 columns = f"{columns[0]} / {columns[1]+1}"
@@ -461,7 +488,7 @@ class Style:
         right: int = None,
         top: int = None,
         bottom: int = None,
-    ) -> "Style":
+    ) -> Style:
         self.rule("position", position)
 
         if left is not None:
@@ -483,7 +510,7 @@ class Style:
         right: int = None,
         top: int = None,
         bottom: int = None,
-    ) -> "Style":
+    ) -> Style:
         self.position("absolute", left=left, right=right, top=top, bottom=bottom)
 
     # #### `Style.relative`
@@ -496,7 +523,7 @@ class Style:
         right: int = None,
         top: int = None,
         bottom: int = None,
-    ) -> "Style":
+    ) -> Style:
         self.position("relative", left=left, right=right, top=top, bottom=bottom)
 
     # ### Animations
@@ -510,7 +537,7 @@ class Style:
         duration: Unit = ms(150),
         timing: str = "linear",
         delay: Unit = ms(0),
-    ) -> "Style":
+    ) -> Style:
         self._transitions.append(
             (
                 property,
@@ -546,7 +573,7 @@ class Style:
         scale_x: float = None,
         scale_y: float = None,
         rotate: Unit = None,
-    ) -> "Style":
+    ) -> Style:
         if translate_x is not None:
             self._transforms["translateX"] = Unit.infer(translate_x)
         if translate_y is not None:
@@ -566,13 +593,11 @@ class Style:
         self.rule("transform", " ".join(transforms))
 
     @style_method
-    def translate(self, x: Unit = None, y: Unit = None) -> "Style":
+    def translate(self, x: Unit = None, y: Unit = None) -> Style:
         self.transform(translate_x=x, translate_y=y)
 
     @style_method
-    def scale(
-        self, scale: float = None, *, x: float = None, y: float = None
-    ) -> "Style":
+    def scale(self, scale: float = None, *, x: float = None, y: float = None) -> Style:
         if scale is not None:
             x = scale
             y = scale
@@ -580,14 +605,50 @@ class Style:
         self.transform(scale_x=x, scale_y=y)
 
     @style_method
-    def rotate(self, rotation: float) -> "Style":
+    def rotate(self, rotation: float) -> Style:
         self.transform(rotate=rotation)
 
+    # #### `Style.animation`
+
+    @style_method
+    def animate(
+        self,
+        animation: Animation,
+        duration: Unit = sec(1),
+        iter: int = 1,
+        timing: str = "linear",
+        direction: str = "normal",
+    ) -> Style:
+        self._animation_configs.append((
+            animation.name, Unit.infer(duration, sec, ms), timing, iter, direction
+        ))
+
+        self._animations.add(animation)
+
+        names = []
+        durations = []
+        timings = []
+        iterations = []
+        directions = []
+
+        for name, duration, timing, iter, direction in self._animation_configs:
+            names.append(name)
+            durations.append(str(duration))
+            timings.append(timing)
+            iterations.append(iter)
+            directions.append(direction)
+
+        self.rule("animation-name", ", ".join(names))
+        self.rule("animation-duration", ", ".join(durations))
+        self.rule("animation-timing-function", ", ".join(timings))
+        self.rule("animation-iteration-count", ", ".join(iterations))
+        self.rule("animation-direction", ", ".join(directions))
+
     # ### Sub-styles
 
     # #### `Style.on`
 
-    def on(self, state) -> "Style":
+    def on(self, state) -> Style:
         selector = self.selector.on(state)
         style = self._children.get(selector.css(), Style(selector))
         self._children[selector.css()] = style
@@ -595,7 +656,7 @@ class Style:
 
     # #### `Style.children`
 
-    def children(self, selector: str = "*", *, nth: int = None) -> "Style":
+    def children(self, selector: str = "*", *, nth: int = None) -> Style:
         selector = self.selector.children(selector, nth=nth)
         style = self._children.get(selector.css(), Style(selector))
         self._children[selector.css()] = style
@@ -615,7 +676,8 @@ class Style:
         if inline:
             return rules
 
-        return f"{self.selector.css()} {{\n{textwrap.indent(rules, 4*' ')}\n}}"
+        selector = self.selector.css() if self.selector is not None else ""
+        return f"{selector} {{\n{textwrap.indent(rules, 4*' ')}\n}}"
 
     # #### `Style.inline`
 
